"""Tushare Equity Profile Model."""

# pylint: disable=unused-argument

from typing import Any, Dict, List, Optional
from datetime import date as dateType

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.equity_info import (
    EquityInfoData,
    EquityInfoQueryParams,
)
from pydantic import Field, field_validator
import pandas as pd


class TushareEquityProfileQueryParams(EquityInfoQueryParams):
    """Tushare Equity Profile Query."""

    __json_schema_extra__ = {"symbol": {"multiple_items_allowed": True}}

    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )

class TushareEquityProfileData(EquityInfoData):
    """Tushare Equity Profile Data."""

    __alias_dict__ = {
        "symbol": "ts_code",
        "stock_exchange": "exchange",
        "long_description": "introduction",
        "ceo": "chairman",
        "company_url": "website",
        "hq_address1": "office",
        "hq_address_city": "city",
        "hq_state": "province",
        "entity_status": "list_status",
        "industry_category": "market",
    }

    @field_validator("employees", mode="before", check_fields=False)
    @classmethod
    def validate_employees(cls, v: Optional[int]) -> Optional[int]:
        """Return 0 if it is nan."""
        if v is None or v == "" or pd.isna(v):
            return 0
        else:
            return int(v)


class TushareEquityProfileFetcher(
    Fetcher[TushareEquityProfileQueryParams, List[TushareEquityProfileData]]
):
    """Tushare Equity Profile fetcher."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> TushareEquityProfileQueryParams:
        """Transform the query."""
        return TushareEquityProfileQueryParams(**params)

    @staticmethod
    async def aextract_data(
        query: TushareEquityProfileQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the raw data from Tushare."""
        # pylint: disable=import-outside-toplevel
        import asyncio  # noqa
        from openbb_core.app.model.abstract.error import OpenBBError
        from openbb_core.provider.utils.errors import EmptyDataError
        from warnings import warn

        api_key = credentials.get("tushare_api_key") if credentials else ""

        symbols = query.symbol.split(",")
        results = []
        messages: list = []

        async def get_one(symbol, api_key: str, use_cache: bool = True) -> None:
            from openbb_tushare.utils.ts_equity_profile import get_equity_profile
            """Get the data for one ticker symbol."""
            try:
                result: dict = {}
                result = get_equity_profile(symbol, api_key=api_key, use_cache=use_cache).to_dict(orient="records")[0]
                if result:
                    results.append(result)
            except Exception as e:
                messages.append(
                    f"Error getting data for {symbol} -> {e.__class__.__name__}: {e}"
                )

        tasks = [get_one(symbol, api_key=api_key, use_cache=query.use_cache) for symbol in symbols]

        await asyncio.gather(*tasks)

        if not results and messages:
            raise OpenBBError("\n".join(messages))

        if not results and not messages:
            raise EmptyDataError("No data was returned for any symbol")

        if results and messages:
            for message in messages:
                warn(message)

        return results

    @staticmethod
    def transform_data(
        query: TushareEquityProfileQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[TushareEquityProfileData]:
        """Transform the data."""
        return [TushareEquityProfileData.model_validate(d) for d in data]



import logging
import pandas as pd
import tushare as ts
from typing import Optional, Literal
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from mysharelib.tools import normalize_symbol
from openbb_tushare import project_name

setup_logger(project_name)

logger = logging.getLogger(__name__)

def get_cash_flow(
        symbol: str, 
        period: Literal["annual", "quarter"] = "annual",
        limit: Optional[int] = 5,
        use_cache: bool = True,
        api_key : Optional[str] = ""
    ) -> pd.DataFrame:
    from mysharelib.blob_cache import BlobCache
    cache = BlobCache(table_name="cash_flow", project=project_name)
    data = cache.load_cached_data(symbol, "quarter", use_cache, get_tushare_data, api_key=api_key)
    if data is None:
        return pd.DataFrame()
    else:
        if period == "annual":
            # Filter rows where end_date ends with "1231"
            data = data[data['end_date'].astype(str).str.endswith('1231')]

        # Apply limit if specified
        if limit is not None:
            data = data.head(limit)
        
        return processing_data(data)
def get_tushare_data(
        symbol: str,
        period: str = "annual",
        api_key : Optional[str] = ""
    ) -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)
    pro = ts.pro_api(tushare_api_key)
    _, _, market = normalize_symbol(symbol)
    if market == 'HK':
        cash_flow_df = pro.hk_cashflow(ts_code=symbol)
    else:
        cash_flow_df = pro.cashflow(ts_code=symbol)
        cash_flow_df = cash_flow_df.drop_duplicates(subset='end_date', keep='first')
    
    return cash_flow_df

def processing_data(cash_flow_df: pd.DataFrame) -> pd.DataFrame:
    from openbb_tushare.utils.helpers import get_fiscal_period
    # logger.info("Processing cash flow data")
    selected_columns = cash_flow_df[['n_cashflow_act', 'n_cashflow_inv_act', 'n_cash_flows_fnc_act']]
    selected_columns = selected_columns.rename(columns={'n_cashflow_act':'net_cash_from_operating_activities', 
                                                        'n_cashflow_inv_act':'net_cash_from_investing_activities',
                                                        'n_cash_flows_fnc_act':'net_cash_from_financing_activities'})

    # Extract year from end_date and create fiscal_year column
    selected_columns.loc[:, 'fiscal_year'] = pd.to_datetime(cash_flow_df['end_date']).dt.year
    selected_columns.loc[:, 'period_ending'] = pd.to_datetime(cash_flow_df['end_date'])
    selected_columns.loc[:, 'fiscal_period'] = cash_flow_df['end_type'].apply(get_fiscal_period)
    return selected_columns


"""Tushare Balance Sheet Model."""

# pylint: disable=unused-argument
import pandas as pd
from datetime import datetime
from typing import Any, Literal, Optional

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.balance_sheet import (
    BalanceSheetData,
    BalanceSheetQueryParams,
)
from openbb_core.provider.utils.descriptions import QUERY_DESCRIPTIONS
from pydantic import Field, field_validator


class TushareBalanceSheetQueryParams(BalanceSheetQueryParams):
    """Tushare Balance Sheet Query.

    Source: https://tushare.pro/document/2?doc_id=36
    """

    __json_schema_extra__ = {
        "period": {
            "choices": ["annual", "quarter"],
        }
    }

    period: Literal["annual", "quarter"] = Field(
        default="annual",
        description=QUERY_DESCRIPTIONS.get("period", ""),
    )
    limit: Optional[int] = Field(
        default=5,
        description=QUERY_DESCRIPTIONS.get("limit", ""),
        le=5,
    )
    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )


class TushareBalanceSheetData(BalanceSheetData):
    """Tushare Balance Sheet Data."""

    @field_validator("period_ending", mode="before", check_fields=False)
    @classmethod
    def date_validate(cls, v):  # pylint: disable=E0213
        """Return datetime object from string."""
        if isinstance(v, str):
            return datetime.strptime(v, "%Y-%m-%d %H:%M:%S").date()
        return v


class TushareBalanceSheetFetcher(
    Fetcher[
        TushareBalanceSheetQueryParams,
        list[TushareBalanceSheetData],
    ]
):
    """Tushare Balance Sheet Fetcher."""

    @staticmethod
    def transform_query(params: dict[str, Any]) -> TushareBalanceSheetQueryParams:
        """Transform the query parameters."""
        return TushareBalanceSheetQueryParams(**params)

    @staticmethod
    def extract_data(
        query: TushareBalanceSheetQueryParams,
        credentials: Optional[dict[str, str]],
        **kwargs: Any,
    ) -> list[dict]:
        """Extract the data from the Tushare endpoints."""
        # pylint: disable=import-outside-toplevel
        from openbb_tushare.utils.ts_balance_sheet import get_balance_sheet
        api_key = credentials.get("tushare_api_key") if credentials else ""

        balance_sheet = get_balance_sheet(query.symbol, query.period, query.limit, query.use_cache, api_key=api_key)

        return balance_sheet.to_dict(orient="records")

    @staticmethod
    def transform_data(
        query: TushareBalanceSheetQueryParams,
        data: list[dict],
        **kwargs: Any,
    ) -> list[TushareBalanceSheetData]:
        """Transform the data."""
        return [TushareBalanceSheetData.model_validate(d) for d in data]




import os
from dotenv import load_dotenv
from typing import Optional, Union

def get_api_key(api_key : Optional[str] = "") -> str:
    if api_key:
        tushare_api_key = api_key
    else:
        load_dotenv() 
        tushare_api_key = os.environ.get("TUSHARE_API_KEY")
    if tushare_api_key is None:
        raise ValueError("TUSHARE_API_KEY environment variable not set.")

    return tushare_api_key

def get_fiscal_period(end_type:int) -> str:
    end_type = int(end_type)
    if end_type == 1:
        return "Q1"
    elif end_type == 2:
        return "Q2"
    elif end_type == 3:
        return "Q3"
    elif end_type == 4:
        return "FY"
    else:
        return "Unknown"



"""Tushare Historical Dividends Model."""

# pylint: disable=unused-argument
from datetime import (
    date as dateType,
    datetime,
)
from typing import Any, Dict, List, Optional
from pydantic import Field, field_validator

from openbb_core.app.model.abstract.error import OpenBBError
from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.historical_dividends import (
    HistoricalDividendsData,
    HistoricalDividendsQueryParams,
)

import logging
from mysharelib.tools import setup_logger
from openbb_tushare import project_name

setup_logger(project_name)
logger = logging.getLogger(__name__)

class TushareHistoricalDividendsQueryParams(HistoricalDividendsQueryParams):
    """Tushare Historical Dividends Query."""
    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )


class TushareHistoricalDividendsData(HistoricalDividendsData):
    """Tushare Historical Dividends Data. All data is split-adjusted."""
    
    @field_validator(
        "ex_dividend_date",
        mode="before",
        check_fields=False,
    )
    @classmethod
    def date_validate(cls, v: str):  # pylint: disable=E0213
        """Validate dates."""
        if not isinstance(v, str):
            return v
        return dateType.fromisoformat(v) if v else None

    
class TushareHistoricalDividendsFetcher(
    Fetcher[
        TushareHistoricalDividendsQueryParams, List[TushareHistoricalDividendsData]
    ]
):
    """Tushare Historical Dividends Fetcher."""

    @staticmethod
    def transform_query(
        params: Dict[str, Any],
    ) -> TushareHistoricalDividendsQueryParams:
        """Transform the query."""
        return TushareHistoricalDividendsQueryParams(**params)

    @staticmethod
    def extract_data(
        query: TushareHistoricalDividendsQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the raw data from Tushare."""
        # pylint: disable=import-outside-toplevel
        from openbb_tushare.utils.ts_historical_dividends import get_dividends
        api_key = credentials.get("tushare_api_key") if credentials else ""

        logger.info(f"Historical Dividends Fetcher start_date:{query.start_date}, end_date:{query.end_date}...")
        return get_dividends(query.symbol, query.start_date, query.end_date, query.use_cache, api_key).to_dict(orient="records")

    @staticmethod
    def transform_data(
        query: TushareHistoricalDividendsQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[TushareHistoricalDividendsData]:
        """Transform the data."""
        #return [TushareHistoricalDividendsData.model_validate(d) for d in data]
        result: List[TushareHistoricalDividendsData] = []
        for d in data:
            result.append(TushareHistoricalDividendsData(**d))
        logger.info(
            "Transformed historical dividends completed.\n"
        )        
        return result



import logging
import pandas as pd
import tushare as ts
from datetime import (
    date as dateType,
    datetime,
)
from typing import Optional, Union
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from mysharelib.tools import normalize_symbol
from mysharelib.table_cache import TableCache
from openbb_tushare import project_name

setup_logger(project_name)

logger = logging.getLogger(__name__)

EQUITY_HISTORY_SCHEMA = {
    "date": "TEXT PRIMARY KEY",
    "open": "REAL",
    "high": "REAL",
    "low": "REAL",
    "close": "REAL",
    "volume": "REAL",
    "vwap": "REAL",
    "change": "REAL",
    "change_percent": "REAL",
    "amount": "REAL"
}

def get_from_cache(
        ts_code: str,
        start_date: Union[dateType, str],
        end_date: Union[dateType, str],
        api_key : str = "",
        period: str = "daily",
        use_cache: bool = True,
        adjust: str = ""
    ) -> pd.DataFrame:
    """
    Retrieves historical equity data from a cache or downloads it from a remote source.
    
    Parameters:
        symbol (str): Stock symbol to fetch data for.
        start_date (str): Start date for fetching data in 'YYYYMMDD' format.
        end_date (str): End date for fetching data in 'YYYYMMDD' format.
        period (str): Data frequency, e.g., "daily", "weekly", "monthly".
        adjust (str): Adjustment type, e.g., "qfq" for forward split, "hfq" for backward split.

    Returns:
        DataFrame: DataFrame containing historical equity data.
    """
    from mysharelib.tools import get_valid_date

    # Retrieve data from cache first
    symbol_b, symbol_f, market = normalize_symbol(ts_code)
    cache = TableCache(EQUITY_HISTORY_SCHEMA, project=project_name, table_name=f"{market}{symbol_b}", primary_key="date")
    
    start_dt = get_valid_date(start_date)
    end_dt = get_valid_date(end_date)

    start = start_dt.strftime("%Y%m%d")
    end = end_dt.strftime("%Y%m%d")
    if use_cache:
        check_cache(symbol=ts_code, cache=cache, api_key=api_key, period=period)
        data_from_cache = cache.fetch_date_range(start, end)
        if not data_from_cache.empty:
            logger.info(f"Getting equity {ts_code} historical data from cache...")
            return data_from_cache

    # If not in cache, download data
    # Download data using AKShare
    data_util_today_df = get_one(ts_code, period=period,api_key=api_key, start_date=start_dt, end_date=end_dt)
    cache.write_dataframe(data_util_today_df)
    
    return cache.fetch_date_range(start, end)

def get_one(
        ts_code : str, 
        start_date: dateType,
        end_date: dateType,
        period: str = "daily",
        use_cache: bool = True, 
        api_key : str = ""
        ) -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)

    pro = ts.pro_api(tushare_api_key)
    symbol_b, symbol, market = normalize_symbol(ts_code)
    df_data = pd.DataFrame()
    if market == 'HK':
        df_data = pro.hk_daily(ts_code=ts_code)
        logger.info(f"Downloaed historical data (HK) {ts_code}: {len(df_data)}.")
    else:
        df_data = pro.daily(ts_code=ts_code)
        logger.info(f"Downloaed historical data {ts_code}: {len(df_data)}.")

    df_data = df_data.rename(columns={'trade_date':'date', 'vol':'volume', "pct_chg":"change_percent"})
    if 'ts_code' in df_data.columns:
        df_data.drop(columns=['ts_code'], inplace=True)
    return df_data

def check_cache(symbol: str, 
        cache: TableCache,
        api_key : str = "",
        period: str = "daily"
        ) -> bool:
    """
    Check if the cache contains the latest data for the given symbol.
    """
    from openbb_tushare.utils.ak_helpers import get_list_date
    from mysharelib.tools import last_closing_day
    
    start = get_list_date(symbol)
    end = last_closing_day()
    cache_df = cache.fetch_date_range(start.strftime("%Y%m%d"), end.strftime("%Y%m%d"))
    cache_df = cache_df.set_index('date')
    cache_df.index = pd.to_datetime(cache_df.index)
    is_cache_valid = cache_df.index.max().date() == last_closing_day()
    if not is_cache_valid:
        logger.warning(f"Cache for {symbol} is not up-to-date. Last date in cache: {cache_df.index.max().date()}, expected: {last_closing_day()}.")
        data_util_today_df = get_one(symbol, period=period, api_key=api_key, start_date=start, end_date=end)
        cache.write_dataframe(data_util_today_df)
    return is_cache_valid


import logging

import pandas as pd
import tushare as ts
from mysharelib.table_cache import TableCache
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from openbb_tushare import project_name

setup_logger(project_name)
logger = logging.getLogger(__name__)

TABLE_SCHEMA = {
    "ts_code": "TEXT PRIMARY KEY",    # Unique identifier (TS代码)
    "name": "TEXT NOT NULL",          # Short name (简称)
    "fullname": "TEXT",               # Full index name (指数全称)
    "market": "TEXT NOT NULL",         # Market identifier (市场)
    "publisher": "TEXT",               # Publisher (发布方)
    "index_type": "TEXT",              # Index style classification (指数风格)
    "category": "TEXT",                # Index category (指数类别)
    "base_date": "TEXT",               # Base period (基期, YYYYMMDD format)
    "base_point": "REAL",              # Base points (基点, floating-point)
    "list_date": "TEXT",               # Listing date (发布日期, YYYYMMDD format)
    "weight_rule": "TEXT",             # Weighting method (加权方式)
    "desc": "TEXT",                    # Description (描述)
    "exp_date": "TEXT",                # Expiry date (终止日期, YYYYMMDD format)
    "currency": "TEXT"                 # Currency type (货币)
}

def get_available_indices(use_cache: bool = True, api_key : str = "") -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)
    cache = TableCache(TABLE_SCHEMA, project=project_name, table_name="indices", primary_key="ts_code")
    if use_cache:
        data = cache.read_dataframe()
        if not data.empty:
            logger.info("Loading indices from cache...")
            return data

    logger.info(f"Generating new indices data...")
    pro = ts.pro_api(tushare_api_key)
    data = pro.index_basic()
    data["currency"] = "CNY"
    cache.write_dataframe(data)
    return data


import logging
import pandas as pd
import tushare as ts
from datetime import (
    date as dateType
)
from typing import Optional
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from openbb_tushare import project_name

setup_logger(project_name)
logger = logging.getLogger(__name__)

def get_dividends(
        symbol: str, 
        start_date: Optional[dateType] = None,
        end_date: Optional[dateType] = None,
        use_cache: bool = True,
        api_key : str = ""
    ) -> pd.DataFrame:
    """
    Get dividend history for a given symbol.
    Args:
        symbol (str): The stock symbol to query.
        start_date (dateType): The start date for the query in YYYY-MM-DD format.
        end_date (dateType): The end date for the query in YYYY-MM-DD format.
        use_cache (bool): Whether to use cached data.
        api_key (str): Tushare API key.
    """
    from mysharelib.blob_cache import BlobCache
    
    cache = BlobCache(table_name="historical_dividends", project=project_name)
    data = cache.load_cached_data(symbol, "annual", use_cache, get_tushare_data, api_key=api_key)
    if start_date is None or end_date is None:
        return data
    else:
        return data[(data['ex_dividend_date'] >= start_date.strftime('%Y%m%d')) & (data['ex_dividend_date'] <= end_date.strftime('%Y%m%d'))].reset_index(drop=True)

def get_tushare_data(
        symbol: str,
        period: str = "annual",
        api_key : Optional[str] = ""
    ) -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)
    pro = ts.pro_api(tushare_api_key)
    div_df = pro.dividend(ts_code=symbol)
    div_df = div_df[div_df['cash_div'] != 0].reset_index(drop=True)
    div_df = div_df.rename(columns={'cash_div':'amount', 'ex_date':'ex_dividend_date'})
    div_df['ex_dividend_date'] = pd.to_datetime(div_df['ex_dividend_date'], format='%Y%m%d')
    return div_df



import logging

import pandas as pd
import tushare as ts
from mysharelib.table_cache import TableCache
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from openbb_tushare import project_name

setup_logger(project_name)
logger = logging.getLogger(__name__)

TABLE_SCHEMA = {
    "ts_code": "TEXT PRIMARY KEY",  # Trading symbol/code 
    "symbol": "TEXT",               # Symbol (e.g. 600000)
    "name": "TEXT",                 # Short name of the asset
    "area": "TEXT",                 # Geographic region
    "industry": "TEXT",             # Industry classification
    "fullname": "TEXT",             # Full legal name
    "enname": "TEXT",               # English name
    "cnspell": "TEXT",              # Chinese phonetic abbreviation
    "market": "TEXT",               # Market classification (e.g. A-shares)
    "exchange": "TEXT",             # Exchange code (e.g. SSE/SZSE)
    "curr_type": "TEXT",            # Currency type (CNY/USD/HKD)
    "list_status": "TEXT",          # Listing status (L/D/P)
    "list_date": "TEXT",            # Listing date (YYYYMMDD)
    "delist_date": "TEXT",          # Delisting date (YYYYMMDD)
    "is_hs": "TEXT",                # Hong Kong Connect eligibility (N/H/S)
    "act_name": "TEXT",             # Actual controller name
    "act_ent_type": "TEXT",         # Controller entity type
    "trade_unit": "TEXT",           # Trading unit (e.g. 100 shares)
    "isin": "TEXT",                 # International Securities Identification Number
}

def get_symbols(use_cache: bool = True, api_key : str = "") -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)

    cache = TableCache(TABLE_SCHEMA, project=project_name, table_name="symbols", primary_key="ts_code")
    if use_cache:
        data = cache.read_dataframe()
        if not data.empty:
            logger.info("Loading symbols from cache...")
            return data

    logger.info(f"Generating symbols ...")
    pro = ts.pro_api(tushare_api_key)
    df_hk = pro.hk_basic()
    df_hk['symbol'] = df_hk['ts_code'].str.replace('.HK', '', regex=False)
    df_hk['exchange'] = 'HKEX'
    df_cn = pro.stock_basic(exchange='', list_status='L', fields='ts_code,symbol,name,area,industry,fullname,enname,cnspell,market,exchange,curr_type,list_status,list_date,delist_date,is_hs,act_name,act_ent_type')
    df_all = pd.concat([df_cn, df_hk], ignore_index=True)
    cache.write_dataframe(df_all)
    return df_all


"""openbb_tushare OpenBB Platform Provider."""

from openbb_core.provider.abstract.provider import Provider
from openbb_tushare.models.available_indices import TushareAvailableIndicesFetcher
from openbb_tushare.models.balance_sheet import TushareBalanceSheetFetcher
from openbb_tushare.models.cash_flow import TushareCashFlowStatementFetcher
from openbb_tushare.models.equity_historical import TushareEquityHistoricalFetcher
from openbb_tushare.models.equity_profile import TushareEquityProfileFetcher
from openbb_tushare.models.equity_quote import TushareEquityQuoteFetcher
from openbb_tushare.models.equity_search import TushareEquitySearchFetcher
from openbb_tushare.models.historical_dividends import TushareHistoricalDividendsFetcher
from openbb_tushare.models.income_statement import TushareIncomeStatementFetcher

# mypy: disable-error-code="list-item"

provider = Provider(
    name="tushare",
    description="Data provider for openbb_tushare.",
    credentials=["api_key"],
    website="https://github.com/finanalyzer/openbb_tushare",
    # Here, we list out the fetchers showing what our provider can get.
    # The dictionary key is the fetcher's name, used in the `router.py`.
    fetcher_dict={
        "AvailableIndices": TushareAvailableIndicesFetcher,
        "BalanceSheet": TushareBalanceSheetFetcher,
        "CashFlowStatement": TushareCashFlowStatementFetcher,
        "EquityHistorical": TushareEquityHistoricalFetcher,
        "EquityInfo": TushareEquityProfileFetcher,
        "EquityQuote": TushareEquityQuoteFetcher,
        "EquitySearch": TushareEquitySearchFetcher,
        "HistoricalDividends": TushareHistoricalDividendsFetcher,
        "IncomeStatement": TushareIncomeStatementFetcher,
    }
)



"""Tushare Equity Quote Model."""

# pylint: disable=unused-argument

from typing import Any, Dict, List, Optional
from warnings import warn

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.equity_quote import (
    EquityQuoteData,
    EquityQuoteQueryParams,
)
from pydantic import Field
import logging
from mysharelib.tools import setup_logger, normalize_symbol
from openbb_tushare import project_name

setup_logger(project_name)
logger = logging.getLogger(__name__)

class TushareEquityQuoteQueryParams(EquityQuoteQueryParams):
    """Tushare Equity Quote Query."""

    __json_schema_extra__ = {"symbol": {"multiple_items_allowed": True}}

    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )


class TushareEquityQuoteData(EquityQuoteData):
    """Tushare Equity Quote Data."""

    __alias_dict__ = {
        "symbol": "ts_code",
    }


class TushareEquityQuoteFetcher(
    Fetcher[TushareEquityQuoteQueryParams, List[TushareEquityQuoteData]]
):
    """Tushare Equity Quote Fetcher."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> TushareEquityQuoteQueryParams:
        """Transform the query."""
        return TushareEquityQuoteQueryParams(**params)

    @staticmethod
    def extract_data(
        query: TushareEquityQuoteQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the raw data from Tushare."""
        # pylint: disable=import-outside-toplevel
        from openbb_tushare.utils.ts_equity_quote import get_one
        import pandas as pd

        api_key = credentials.get("tushare_api_key") if credentials else ""
        symbols = query.symbol.split(",")
        all_data = []

        for symbol in symbols:
            try:        
                data = get_one(symbol, query.use_cache, api_key=api_key)
                all_data.append(data.to_dict(orient="records")[0])
                
            except Exception as e:
                print(f"Error fetching data for symbol {symbol}: {e}")
                continue

        return all_data

    @staticmethod
    def transform_data(
        query: TushareEquityQuoteQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[TushareEquityQuoteData]:
        """Transform the data."""
        return [TushareEquityQuoteData.model_validate(d) for d in data]



"""Tushare Equity Search Model."""

from typing import Any, Dict, List, Optional

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.equity_search import (
    EquitySearchData,
    EquitySearchQueryParams,
)
from openbb_core.provider.utils.descriptions import (
    DATA_DESCRIPTIONS,
    QUERY_DESCRIPTIONS,
)
from pydantic import Field


class TushareEquitySearchQueryParams(EquitySearchQueryParams):
    """Tushare Equity Search Query.

    Source: https://tushare.pro/document/2?doc_id=25
    """

    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )
    limit: Optional[int] = Field(
        default=10000,
        description=QUERY_DESCRIPTIONS.get("limit", ""),
    )


class TushareEquitySearchData(EquitySearchData):
    """Tushare Equity Search Data."""


class TushareEquitySearchFetcher(
    Fetcher[
        TushareEquitySearchQueryParams,
        List[TushareEquitySearchData],
    ]
):
    """Transform the query, extract and transform the data from the Tushare endpoints."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> TushareEquitySearchQueryParams:
        """Transform the query."""
        return TushareEquitySearchQueryParams(**params)

    @staticmethod
    async def aextract_data(
        query: TushareEquitySearchQueryParams,  # pylint: disable=unused-argument
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Return the raw data from the Tushare endpoint."""

        from openbb_tushare.utils.ts_equity_search import get_symbols
        api_key = credentials.get("tushare_api_key") if credentials else ""

        return get_symbols(query.use_cache, api_key=api_key).to_dict(orient="records")

    @staticmethod
    def transform_data(
        query: TushareEquitySearchQueryParams, data: Dict, **kwargs: Any
    ) -> List[TushareEquitySearchData]:
        """Transform the data to the standard format."""

        return [TushareEquitySearchData.model_validate(d) for d in data]



"""
Helper functions using AKShare API.
"""
from datetime import (
    date as dateType,
    datetime,
    timedelta
)
import pandas as pd
import akshare as ak
from mysharelib.tools import normalize_symbol

def get_list_date(code: str) -> dateType:
    """
    查询 A 股/港股的上市日期
    code: 对于 A 股请直接给 6 位数字，如 000001
          港股请加 .HK，如 00700.HK
          美股请加 .US，如 AAPL.US
    return: yyyy-mm-dd 字符串
    """
    symbol_b, symbol_f, market = normalize_symbol(code)
    list_date = datetime.now().date()
    if symbol_f.endswith(".HK"):
        # 港股, 2004-06-16
        df = ak.stock_hk_security_profile_em(symbol=code.replace(".HK", ""))
        list_date = datetime.strptime(df.iloc[0]["上市日期"][:10], "%Y-%m-%d")
    else:
        # A 股, 19910403
        df = ak.stock_individual_info_em(symbol=symbol_b)
        list_date = datetime.strptime(str(df[df["item"] == "上市时间"]["value"].iat[0]), "%Y%m%d")

    return list_date.date()





"""Tushare Available Indices Model."""

# pylint: disable=unused-argument

from typing import Any, Dict, List, Optional
from datetime import (
    date as dateType
)

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.available_indices import (
    AvailableIndicesData,
    AvailableIndicesQueryParams,
)
from pydantic import Field


class TushareAvailableIndicesQueryParams(AvailableIndicesQueryParams):
    """Tushare Available Indices Query.

    Source: https://tushare.pro/document/2?doc_id=94
    """
    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )

class TushareAvailableIndicesData(AvailableIndicesData):
    """Tushare Available Indices Data."""

class TushareAvailableIndicesFetcher(
    Fetcher[
        TushareAvailableIndicesQueryParams,
        List[TushareAvailableIndicesData],
    ]
):
    """Transform the query, extract and transform the data from the Tushare endpoints."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> TushareAvailableIndicesQueryParams:
        """Transform the query params."""
        return TushareAvailableIndicesQueryParams(**params)

    @staticmethod
    def extract_data(
        query: TushareAvailableIndicesQueryParams,  # pylint disable=unused-argument
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the data."""
        from openbb_tushare.utils.ts_available_indices import get_available_indices
        api_key = credentials.get("tushare_api_key") if credentials else ""

        return get_available_indices(query.use_cache, api_key=api_key).to_dict(orient="records")

    @staticmethod
    def transform_data(
        query: TushareAvailableIndicesQueryParams, data: List[Dict], **kwargs: Any
    ) -> List[TushareAvailableIndicesData]:
        """Return the transformed data."""
        return [TushareAvailableIndicesData.model_validate(d) for d in data]



import logging
import pandas as pd
import tushare as ts
from datetime import (
    date as dateType,
    datetime,
)
from typing import Optional, Union
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from mysharelib.tools import normalize_symbol
from mysharelib.table_cache import TableCache
from openbb_tushare import project_name

setup_logger(project_name)

logger = logging.getLogger(__name__)

EQUITY_INFO_SCHEMA = {
    "ts_code": "TEXT PRIMARY KEY",        # 证券代码 (Security ID)
    "name": "TEXT",          # 公司常用名称 (Common Name)
    "isin": "TEXT",                # 国际证券识别号码 (ISIN Code)
    "exchange": "TEXT",          # 证券交易所 (Exchange)
    "introduction": "TEXT",               # 公司描述 (Description)
    "chairman": "TEXT",                   # 董事长 (Chairman)
    "website": "TEXT",                    # 公司网站 (Website URL)
    "office": "TEXT",                     # 总部地址 (Headquarters)
    "city": "TEXT",                       # 城市 (City)
    "province": "TEXT",                   # 省份/州 (Province/State)
    "employees": "INTEGER",               # 员工数量 (Employee Count)
    "list_status": "TEXT",                # 上市状态 (List Status)
    "market": "TEXT",                     # 行业分类 (Industry/Market)
    "com_id": "TEXT",              # 统一社会信用代码 (Business ID)
    "manager": "TEXT",                    # 总经理 (General Manager)
    "secretary": "TEXT",                  # 董事会秘书 (Board Secretary)
    "reg_capital": "REAL",                # 注册资本 (Registered Capital)
    "setup_date": "TEXT",                 # 注册日期 (Registration Date)
    "email": "TEXT",                      # 邮箱 (Email)
    "ann_date": "TEXT",                   # 公告日期 (Announcement Date)
    "business_scope": "TEXT",             # 经营范围 (Business Scope)
    "main_business": "TEXT",              # 主营业务 (Main Business)
    "fullname": "TEXT",          # 公司全称 (Full Legal Name)
    "enname": "TEXT",                     # 英文名称 (English Name)
    "cn_spell": "TEXT",                   # 拼音 (Chinese Spelling)
    "list_date": "TEXT",         # 上市日期 (Listing Date)
    "delist_date": "TEXT",                # 退市日期 (Delisting Date)
    "trade_unit": "REAL",                 # 交易单位 (Trading Unit)
    "curr_type": "TEXT"          # 货币代码 (Currency)
}

def get_hk_data(ts_code: str, pro, cache: TableCache) -> pd.DataFrame:
    a_fields = {
        'ts_code': ts_code,
        'com_id': '',
        'exchange': '',
        'chairman': '',
        'manager': '',
        'secretary': '',
        'reg_capital': 0.0,
        'setup_date': '',
        'province': '',
        'city': '',
        'introduction': '',
        'website': '',
        'email': '',
        'office': '',
        'employees': 0,
        'main_business': '',
        'business_scope': ''
    }

    data_hk = pro.hk_basic(ts_code=ts_code)
    if data_hk.empty:
        logger.warning(f"No equity profile data found for HK stock {ts_code}.")
        return pd.DataFrame()

    data_a = pd.DataFrame([a_fields])
    combined_data = pd.merge(data_hk, data_a, on=['ts_code'], how='outer')
    cache.update_or_insert(combined_data)
    return combined_data

def get_ss_data(ts_code: str, pro, cache: TableCache) -> pd.DataFrame:
    csv_fields = "ts_code,com_name,com_id,exchange,chairman,manager,secretary,reg_capital,setup_date,province,city,introduction,website,email,office,employees,main_business,business_scope"
    data = pro.stock_company(ts_code=ts_code, fields=csv_fields)
    if data.empty:
        logger.warning(f"No equity profile data found for HK stock {ts_code}.")
        return data

    data = data.rename(columns={'com_name': 'name'})

    hk_fields = {
        'ts_code': ts_code,
        'fullname': '',
        'enname': '',
        'cn_spell': '',
        'market': '',
        'list_status': '',
        'list_date': '',
        'delist_date': '',
        'trade_unit': 0.0,
        'isin': '',
        'curr_type': ''
    }
    data_hk = pd.DataFrame([hk_fields])
    combined_data = pd.merge(data_hk, data, on=['ts_code'], how='outer')
    cache.update_or_insert(combined_data)
    return combined_data

def get_equity_profile(ts_code: str, api_key: str = "", use_cache: bool = True) -> pd.DataFrame:
    """
    Retrieves equity profile data from a cache or downloads it from the data source.
    
    Parameters:
        ts_code (str): Tushare stock symbol to fetch data for.
        use_cache (bool): Whether to use cached data or download fresh data.
        api_key (str): Tushare API key for authentication.

    Returns:
        DataFrame: DataFrame containing equity profile data.
    """
    cache = TableCache(EQUITY_INFO_SCHEMA, project=project_name, table_name="equity_profile", primary_key="ts_code")
    if use_cache:
        filters = {'ts_code': ts_code}
        data = cache.read_rows(filters)

        if not data.empty:
            logger.info(f"Loading equity profile {ts_code} from cache...")
            return data

    tushare_api_key = get_api_key(api_key)
    pro = ts.pro_api(tushare_api_key)
    symbol_b, symbol, market = normalize_symbol(ts_code)
    df_data = pd.DataFrame()
    if market == 'HK':
        df_data = get_hk_data(ts_code, pro, cache)
    else:
        df_data = get_ss_data(ts_code, pro, cache)

    return df_data

    


"""openbb_tushare router command example."""

import requests
from openbb_core.app.model.command_context import CommandContext
from openbb_core.app.model.obbject import OBBject
from openbb_core.app.provider_interface import (ExtraParams, ProviderChoices,
                                                StandardParams)
from openbb_core.app.query import Query
from openbb_core.app.router import Router
from pydantic import BaseModel

router = Router(prefix="")


@router.command(methods=["GET"])
async def get_example(symbol: str = "AAPL") -> OBBject[dict]:
    """Get options data."""
    base_url = "https://www.cboe.com/education/tools/trade-optimizer/symbol-info"

    response = requests.get(base_url + f"?symbol={symbol}", timeout=5).json()
    return OBBject(results=response["details"])


@router.command(methods=["POST"])
async def post_example(
    data: dict,
    bid_col: str = "bid",
    ask_col: str = "ask",
) -> OBBject[dict]:
    """Calculate mid and spread."""
    bid = data[bid_col]
    ask = data[ask_col]
    mid = (bid + ask) / 2
    spread = ask - bid

    return OBBject(results={"mid": mid, "spread": spread})


# pylint: disable=unused-argument
@router.command(model="Example")
async def model_example(
    cc: CommandContext,
    provider_choices: ProviderChoices,
    standard_params: StandardParams,
    extra_params: ExtraParams,
) -> OBBject[BaseModel]:
    """Example Data."""
    return await OBBject.from_query(Query(**locals()))



"""Tushare Income Statement Model."""

# pylint: disable=unused-argument
import pandas as pd
from datetime import (
    date as dateType,
    datetime,
)
from typing import Any, Dict, List, Literal, Optional

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.income_statement import (
    IncomeStatementData,
    IncomeStatementQueryParams,
)
from openbb_core.provider.utils.descriptions import QUERY_DESCRIPTIONS
from pydantic import Field, field_validator


class TushareIncomeStatementQueryParams(IncomeStatementQueryParams):
    """Tushare Income Statement Query.

    Source: https://tushare.pro/document/2?doc_id=33
    """

    __json_schema_extra__ = {
        "period": {
            "choices": ["annual", "quarter"],
        }
    }

    period: Literal["annual", "quarter"] = Field(
        default="annual",
        description=QUERY_DESCRIPTIONS.get("period", ""),
    )
    use_cache: bool = Field(
        default=True,
        description="Whether to use a cached request. The quote is cached for one hour.",
    )


class TushareIncomeStatementData(IncomeStatementData):
    """Tushare Income Statement Data."""

    @field_validator("period_ending", mode="before", check_fields=False)
    @classmethod
    def date_validate(cls, v):
        """Return datetime object from string."""
        if isinstance(v, str):
            return datetime.strptime(v, "%Y-%m-%d %H:%M:%S").date()
        return v


class TushareIncomeStatementFetcher(
    Fetcher[
        TushareIncomeStatementQueryParams,
        List[TushareIncomeStatementData],
    ]
):
    """Transform the query, extract and transform the data from the Tushare endpoints."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> TushareIncomeStatementQueryParams:
        """Transform the query params."""
        return TushareIncomeStatementQueryParams(**params)

    @staticmethod
    async def extract_data(
        query: TushareIncomeStatementQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Return the raw data from the Tushare endpoint."""
        from openbb_tushare.utils.ts_income_statement import get_income_statement
        api_key = credentials.get("tushare_api_key") if credentials else ""

        income_statement = get_income_statement(query.symbol, query.period, query.limit, query.use_cache, api_key=api_key)

        return income_statement.to_dict(orient="records")
    @staticmethod
    def transform_data(
        query: TushareIncomeStatementQueryParams, data: List[Dict], **kwargs: Any
    ) -> List[TushareIncomeStatementData]:
        """Return the transformed data."""
        for result in data:
            result.pop("symbol", None)
            result.pop("cik", None)
        return [TushareIncomeStatementData.model_validate(d) for d in data]


"""
Helper functions using Tushare API.
"""
import tushare as ts
from openbb_tushare.utils.helpers import get_api_key

def get_list_date(ts_code: str, api_key : str = "") -> str:
    """
    ts_code: 000001.SZ / 600000.SH 这种格式
    该函数只支持A股
    """
    tushare_api_key = get_api_key(api_key)

    pro = ts.pro_api(tushare_api_key)
    df = pro.stock_basic(ts_code=ts_code,
                         fields="ts_code,name,list_date")
    return df.iloc[0]["list_date"]   # 形如 19910129


"""openbb_tushare OpenBB Platform extension."""
from pathlib import Path
from typing import List, Optional, Union

from openbb_core.app.static.app_factory import BaseApp as _BaseApp
from openbb_core.app.static.app_factory import create_app as _create_app
from openbb_core.app.static.package_builder import \
    PackageBuilder as _PackageBuilder

_this_dir = Path(__file__).parent.resolve()
_PackageBuilder(_this_dir).auto_build()


def build(
    modules: Optional[Union[str, List[str]]] = None,
    lint: bool = True,
    verbose: bool = False,
) -> None:
    """Build extension modules.

    Parameters
    ----------
    modules : Optional[List[str]], optional
        The modules to rebuild, by default None
        For example: "/news" or ["/news", "/crypto"]
        If None, all modules are rebuilt.
    lint : bool, optional
        Whether to lint the code, by default True
    verbose : bool, optional
        Enable/disable verbose mode
    """
    _PackageBuilder(_this_dir, lint, verbose).build(modules)


try:
    # pylint: disable=import-outside-toplevel
    from openbb_tushare.package.__extensions__ import Extensions as _Extensions

    obb: Union[_BaseApp, _Extensions] = _create_app(_Extensions)  # type: ignore
    sdk = obb
except (ImportError, ModuleNotFoundError):
    print("Failed to import extensions.")
    obb = sdk = _create_app()  # type: ignore



import logging
import pandas as pd
import tushare as ts
from mysharelib.tools import setup_logger
from openbb_tushare.utils.helpers import get_api_key
from mysharelib.tools import normalize_symbol
from openbb_tushare import project_name

setup_logger(project_name)

logger = logging.getLogger(__name__)

def get_one(ts_code : str, use_cache: bool = True, api_key : str = "") -> pd.DataFrame:
    tushare_api_key = get_api_key(api_key)

    logger.info(f"Getting equity equote data...")
    _, _, market = normalize_symbol(ts_code)
    df_data = pd.DataFrame()
    if market == 'HK':
        pro = ts.pro_api(tushare_api_key)
        df_data = pro.rt_hk_k(ts_code)
        # For HK market: select required columns and rename
        df_data = df_data[['ts_code', 'open', 'high', 'low', 'close', 'vol', 'pre_close']]
        df_data = df_data.rename(columns={'vol': 'volume', 'pre_close': 'prev_close'})
    else:
        ts.set_token(tushare_api_key)
        df_data = ts.realtime_quote(ts_code)
        if df_data is None or df_data.empty:
            logger.warning(f"No data returned from tushare for {ts_code}")
            return pd.DataFrame()
        
        # For non-HK markets: select required columns and rename
        df_data = df_data[['TS_CODE','NAME','BID','ASK','PRICE','OPEN','HIGH','LOW','VOLUME','PRE_CLOSE']]
        df_data = df_data.rename(columns={'TS_CODE':'ts_code', 'NAME':'name', 'BID':'bid', 
                                          'ASK':'ask', 'PRICE': 'last_price', 'OPEN':'open', 
                                          'HIGH':'high', 'LOW':'low', 'VOLUME':'volume', 'PRE_CLOSE': 'prev_close'})
    return df_data



import logging
from mysharelib.tools import setup_logger

project_name = __name__

#setup_logger(project_name)
#logger = logging.getLogger(project_name)

#logger.info(f"Current openbb_tushare module name: {project_name}")
